@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2012 Nebula, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -12,20 +14,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from collections import OrderedDict
-import logging
-import operator
+import sys
 
-from django.conf import settings
-from django.template.loader import render_to_string
 from django.template import TemplateSyntaxError
-from django.utils import module_loading
+from django.template.loader import render_to_string
+from django.utils.datastructures import SortedDict
 
 from horizon import exceptions
 from horizon.utils import html
-from horizon.utils import settings as utils_settings
-
-LOG = logging.getLogger(__name__)
 
 SEPARATOR = "__"
 CSS_TAB_GROUP_CLASSES = ["nav", "nav-tabs", "ajax-tabs"]
@@ -34,17 +30,14 @@ CSS_DISABLED_TAB_CLASSES = ["disabled"]
 
 
 class TabGroup(html.HTMLElement):
-    """A container class which knows how to manage and render Tab objects.
+    """
+    A container class which knows how to manage and render
+    :class:`~horizon.tabs.Tab` objects.
 
     .. attribute:: slug
 
         The URL slug and pseudo-unique identifier for this tab group.
 
-    .. attribute:: tabs
-
-       A list of :class:`.Tab` classes. Tabs specified here are displayed
-       in the order of the list.
-
     .. attribute:: template_name
 
         The name of the template which will be used to render this tab group.
@@ -56,11 +49,6 @@ class TabGroup(html.HTMLElement):
         across requests for a given user. (State storage is all done
         client-side.)
 
-    .. attribute:: show_single_tab
-
-        Boolean to control whether the tab bar is shown when the tab group
-        has only one tab. Default: ``False``
-
     .. attribute:: param_name
 
         The name of the GET request parameter which will be used when
@@ -90,7 +78,6 @@ class TabGroup(html.HTMLElement):
     template_name = "horizon/common/_tab_group.html"
     param_name = 'tab'
     sticky = False
-    show_single_tab = False
     _selected = None
     _active = None
 
@@ -103,102 +90,60 @@ class TabGroup(html.HTMLElement):
         return self._active
 
     def __init__(self, request, **kwargs):
-        super().__init__()
+        super(TabGroup, self).__init__()
         if not hasattr(self, "tabs"):
             raise NotImplementedError('%s must declare a "tabs" attribute.'
                                       % self.__class__)
-        if not self.slug:
-            raise NotImplementedError('%s must declare a "slug" attribute.'
-                                      % self.__class__)
         self.request = request
         self.kwargs = kwargs
         self._data = None
-        self._tabs = self._load_tabs(request)
+        tab_instances = []
+        for tab in self.tabs:
+            tab_instances.append((tab.slug, tab(self, request)))
+        self._tabs = SortedDict(tab_instances)
         if self.sticky:
             self.attrs['data-sticky-tabs'] = 'sticky'
         if not self._set_active_tab():
             self.tabs_not_available()
 
-    def _load_tabs(self, request):
-        tabs = tuple(self.tabs)
-        tabs += self._load_tabs_from_config()
-        return OrderedDict([(tab.slug, tab(self, request))
-                            for tab in tabs])
-
-    def _load_tabs_from_config(self):
-        my_name = '.'.join([self.__class__.__module__,
-                            self.__class__.__name__])
-        horizon_config = settings.HORIZON_CONFIG.get('extra_tabs', {})
-        tabs_config = [self._load_tab_config(tab_config, my_name)
-                       for tab_config in horizon_config.get(my_name, [])]
-        tabs_config = [t for t in tabs_config if t]
-        tabs_config.sort(key=operator.itemgetter(0))
-        LOG.debug('Loaded extra tabs for %s: %s',
-                  my_name, tabs_config)
-        return tuple(x[1] for x in tabs_config)
-
-    @staticmethod
-    def _load_tab_config(tab_config, my_name):
-        if isinstance(tab_config, str):
-            tab_config = (0, tab_config)
-        if not isinstance(tab_config, (tuple, list)):
-            LOG.error('Extra tab definition must be a string or '
-                      'a tuple/list (tab group "%s")', my_name)
-            return
-        if len(tab_config) != 2:
-            LOG.error('Entry must be a format of (priority, tab class) '
-                      '(tab group "%s")', my_name)
-            return
-        priority, tab_class = tab_config
-        if not isinstance(priority, int):
-            LOG.error('Priority of tab entry must be an integer '
-                      '(tab group "%s")', my_name)
-            return
-        try:
-            class_ = module_loading.import_string(tab_class)
-        except ImportError:
-            LOG.error('Tab class "%s" is not found (tab group "%s")',
-                      tab_class, my_name)
-            return
-        return priority, class_
-
     def __repr__(self):
         return "<%s: %s>" % (self.__class__.__name__, self.slug)
 
     def load_tab_data(self):
-        """Preload all data that for the tabs that will be displayed."""
+        """
+        Preload all data that for the tabs that will be displayed.
+        """
         for tab in self._tabs.values():
             if tab.load and not tab.data_loaded:
                 try:
                     tab._data = tab.get_context_data(self.request)
-                except Exception:
+                except:
                     tab._data = False
                     exceptions.handle(self.request)
 
     def get_id(self):
-        """Returns the id for this tab group.
-
-        Defaults to the value of the tab
+        """
+        Returns the id for this tab group. Defaults to the value of the tab
         group's :attr:`horizon.tabs.Tab.slug`.
         """
         return self.slug
 
     def get_default_classes(self):
-        """Returns a list of the default classes for the tab group.
-
-        Defaults to ``["nav", "nav-tabs", "ajax-tabs"]``.
         """
-        default_classes = super().get_default_classes()
+        Returns a list of the default classes for the tab group. Defaults to
+        ``["nav", "nav-tabs", "ajax-tabs"]``.
+        """
+        default_classes = super(TabGroup, self).get_default_classes()
         default_classes.extend(CSS_TAB_GROUP_CLASSES)
         return default_classes
 
     def tabs_not_available(self):
-        """The fallback handler if no tabs are either allowed or enabled.
-
+        """
         In the event that no tabs are either allowed or enabled, this method
         is the fallback handler. By default it's a no-op, but it exists
         to make redirecting or raising exceptions possible for subclasses.
         """
+        pass
 
     def _set_active_tab(self):
         marked_active = None
@@ -224,15 +169,15 @@ class TabGroup(html.HTMLElement):
         return marked_active
 
     def render(self):
-        """Renders the HTML output for this tab group."""
+        """ Renders the HTML output for this tab group. """
         return render_to_string(self.template_name, {"tab_group": self})
 
     def get_tabs(self):
-        """Returns a list of the allowed tabs for this tab group."""
-        return [tab for tab in self._tabs.values() if tab._allowed]
+        """ Returns a list of the allowed tabs for this tab group. """
+        return filter(lambda tab: tab._allowed, self._tabs.values())
 
     def get_tab(self, tab_name, allow_disabled=False):
-        """Returns a specific tab from this tab group.
+        """ Returns a specific tab from this tab group.
 
         If the tab is not allowed or not enabled this method returns ``None``.
 
@@ -245,10 +190,10 @@ class TabGroup(html.HTMLElement):
         return None
 
     def get_loaded_tabs(self):
-        return [tab for tab in self._tabs.values() if self.get_tab(tab.slug)]
+        return filter(lambda t: self.get_tab(t.slug), self._tabs.values())
 
     def get_selected_tab(self):
-        """Returns the tab specific by the GET request parameter.
+        """ Returns the tab specific by the GET request parameter.
 
         In the event that there is no GET request parameter, the value
         of the query parameter is invalid, or the tab is not allowed/enabled,
@@ -256,17 +201,16 @@ class TabGroup(html.HTMLElement):
         """
         selected = self.request.GET.get(self.param_name, None)
         if selected:
-            try:
-                tab_group, tab_name = selected.split(SEPARATOR)
-            except ValueError:
-                return None
+            tab_group, tab_name = selected.split(SEPARATOR)
             if tab_group == self.get_id():
                 self._selected = self.get_tab(tab_name)
         return self._selected
 
 
 class Tab(html.HTMLElement):
-    """A reusable interface for constructing a tab within a TabGroup.
+    """
+    A reusable interface for constructing a tab within a
+    :class:`~horizon.tabs.TabGroup`.
 
     .. attribute:: name
 
@@ -297,42 +241,31 @@ class Tab(html.HTMLElement):
 
         Read-only access to determine whether or not this tab's data should
         be loaded immediately.
-
-    .. attribute:: permissions
-
-        A list of permission names which this tab requires in order to be
-        displayed. Defaults to an empty list (``[]``).
     """
     name = None
     slug = None
     preload = True
     _active = None
-    permissions = []
-    policy_rules = None
 
-    def __init__(self, tab_group, request=None, policy_rules=None):
-        super().__init__()
+    def __init__(self, tab_group, request=None):
+        super(Tab, self).__init__()
         # Priority: constructor, class-defined, fallback
         if not self.name:
             raise ValueError("%s must have a name." % self.__class__.__name__)
+        self.name = unicode(self.name)  # Force unicode.
         if not self.slug:
             raise ValueError("%s must have a slug." % self.__class__.__name__)
         self.tab_group = tab_group
         self.request = request
         if request:
-            self._allowed = self.allowed(request) and (
-                self._has_permissions(request))
+            self._allowed = self.allowed(request)
             self._enabled = self.enabled(request)
-        self.policy_rules = policy_rules or []
 
     def __repr__(self):
         return "<%s: %s>" % (self.__class__.__name__, self.slug)
 
-    def _has_permissions(self, request):
-        return request.user.has_perms(self.permissions)
-
     def is_active(self):
-        """Method to access whether or not this tab is the active tab."""
+        """ Method to access whether or not this tab is the active tab. """
         if self._active is None:
             self.tab_group._set_active_tab()
         return self._active
@@ -353,10 +286,10 @@ class Tab(html.HTMLElement):
         return getattr(self, "_data", None) is not None
 
     def render(self):
-        """Renders the tab to HTML.
-
+        """
+        Renders the tab to HTML using the
         :meth:`~horizon.tabs.Tab.get_context_data` method and
-        the :meth:`~horizon.tabs.Tab.get_template_name` method are called.
+        the :meth:`~horizon.tabs.Tab.get_template_name` method.
 
         If :attr:`~horizon.tabs.Tab.preload` is ``False`` and ``force_load``
         is not ``True``, or
@@ -370,14 +303,15 @@ class Tab(html.HTMLElement):
             context = self.data
         except exceptions.Http302:
             raise
-        except Exception as e:
-            raise TemplateSyntaxError from e
+        except:
+            exc_type, exc_value, exc_traceback = sys.exc_info()
+            raise TemplateSyntaxError, exc_value, exc_traceback
         return render_to_string(self.get_template_name(self.request), context)
 
     def get_id(self):
-        """Returns the id for this tab.
-
-        Defaults to ``"{{ tab_group.slug }}__{{ tab.slug }}"``.
+        """
+        Returns the id for this tab. Defaults to
+        ``"{{ tab_group.slug }}__{{ tab.slug }}"``.
         """
         return SEPARATOR.join([self.tab_group.slug, self.slug])
 
@@ -385,10 +319,10 @@ class Tab(html.HTMLElement):
         return "=".join((self.tab_group.param_name, self.get_id()))
 
     def get_default_classes(self):
-        """Returns a list of the default classes for the tab.
-
-        Defaults to and empty list (``[]``), however additional classes may
-        be added depending on the state of the tab as follows:
+        """
+        Returns a list of the default classes for the tab. Defaults to
+        and empty list (``[]``), however additional classes may be added
+        depending on the state of the tab as follows:
 
         If the tab is the active tab for the tab group, in which
         the class ``"active"`` will be added.
@@ -396,7 +330,7 @@ class Tab(html.HTMLElement):
         If the tab is not enabled, the classes the class ``"disabled"``
         will be added.
         """
-        default_classes = super().get_default_classes()
+        default_classes = super(Tab, self).get_default_classes()
         if self.is_active():
             default_classes.extend(CSS_ACTIVE_TAB_CLASSES)
         if not self._enabled:
@@ -404,7 +338,8 @@ class Tab(html.HTMLElement):
         return default_classes
 
     def get_template_name(self, request):
-        """Returns the name of the template to be used for rendering this tab.
+        """
+        Returns the name of the template to be used for rendering this tab.
 
         By default it returns the value of the ``template_name`` attribute
         on the ``Tab`` class.
@@ -415,18 +350,18 @@ class Tab(html.HTMLElement):
                                  % self.__class__.__name__)
         return self.template_name
 
-    def get_context_data(self, request, **kwargs):
-        """Return a dictionary of context data used to render the tab.
-
-        Required.
+    def get_context_data(self, request):
         """
-        return kwargs
+        This method should return a dictionary of context data used to render
+        the tab. Required.
+        """
+        raise NotImplementedError("%s needs to define a get_context_data "
+                                  "method." % self.__class__.__name__)
 
     def enabled(self, request):
-        """Determines whether or not the tab should be accessible.
-
-        For example, the tab should be rendered into the HTML
-        on load and respond to a click event.
+        """
+        Determines whether or not the tab should be accessible
+        (e.g. be rendered into the HTML on load and respond to a click event).
 
         If a tab returns ``False`` from ``enabled`` it will ignore the value
         of ``preload`` and only render the HTML of the tab after being clicked.
@@ -436,32 +371,21 @@ class Tab(html.HTMLElement):
         return True
 
     def allowed(self, request):
-        """Determines whether or not the tab is displayed.
+        """
+        Determines whether or not the tab is displayed.
 
         Tab instances can override this method to specify conditions under
         which this tab should not be shown at all by returning ``False``.
+
+        The default behavior is to return ``True`` for all cases.
         """
-        if not self.policy_rules:
-            return True
-
-        policy_check = utils_settings.import_setting("POLICY_CHECK_FUNCTION")
-
-        if policy_check:
-            return policy_check(self.policy_rules, request)
         return True
 
-    def post(self, request, *args, **kwargs):
-        """Handles POST data sent to a tab.
-
-        Tab instances can override this method to have tab-specific POST logic
-        without polluting the TabView code.
-
-        The default behavior is to ignore POST data.
-        """
-
 
 class TableTab(Tab):
-    """A Tab class which knows how to deal with DataTable classes inside of it.
+    """
+    A :class:`~horizon.tabs.Tab` class which knows how to deal with
+    :class:`~horizon.tables.DataTable` classes rendered inside of it.
 
     This distinct class is required due to the complexity involved in handling
     both dynamic tab loading, dynamic table updating and table actions all
@@ -476,30 +400,25 @@ class TableTab(Tab):
         need to define a corresponding ``get_{{ table_name }}_data`` method
         as with :class:`~horizon.tables.MultiTableView`.
     """
-    table_classes = []
+    table_classes = None
 
     def __init__(self, tab_group, request):
-        super().__init__(tab_group, request)
+        super(TableTab, self).__init__(tab_group, request)
         if not self.table_classes:
             class_name = self.__class__.__name__
             raise NotImplementedError("You must define a table_class "
                                       "attribute on %s" % class_name)
         # Instantiate our table classes but don't assign data yet
-        if self._allowed:
-            table_instances = [(table._meta.name,
-                                table(request, **tab_group.kwargs))
-                               for table in self.table_classes]
-        else:
-            # When a corresponding tab is not allowed, there is no need to
-            # instantiate table classes (Related to bug 1791296).
-            table_instances = []
-        self._tables = OrderedDict(table_instances)
+        table_instances = [(table._meta.name,
+                            table(request, **tab_group.kwargs))
+                           for table in self.table_classes]
+        self._tables = SortedDict(table_instances)
         self._table_data_loaded = False
 
     def load_table_data(self):
-        """Calls the ``get_{{ table_name }}_data`` methods for each table class.
-
-        When returning, the loaded data is set on the tables.
+        """
+        Calls the ``get_{{ table_name }}_data`` methods for each table class
+        and sets the data on the tables.
         """
         # We only want the data to be loaded once, so we track if we have...
         if not self._table_data_loaded:
@@ -509,28 +428,22 @@ class TableTab(Tab):
                 data_func = getattr(self, func_name, None)
                 if data_func is None:
                     cls_name = self.__class__.__name__
-                    raise NotImplementedError(
-                        "You must define a %(func_name)s method on"
-                        " %(cls_name)s."
-                        % {'func_name': func_name, 'cls_name': cls_name})
-
+                    raise NotImplementedError("You must define a %s method "
+                                              "on %s." % (func_name, cls_name))
                 # Load the data.
                 table.data = data_func()
-                table._meta.has_prev_data = self.has_prev_data(table)
-                table._meta.has_more_data = self.has_more_data(table)
             # Mark our data as loaded so we don't run the loaders again.
             self._table_data_loaded = True
 
-    def get_context_data(self, request, **kwargs):
-        """Adds a ``{{ table_name }}_table`` item to the context for each table.
-
-        The target tables are specified by
-        the :attr:`~horizon.tabs.TableTab.table_classes` attribute.
+    def get_context_data(self, request):
+        """
+        Adds a ``{{ table_name }}_table`` item to the context for each table
+        in the :attr:`~horizon.tabs.TableTab.table_classes` attribute.
 
         If only one table class is provided, a shortcut ``table`` context
         variable is also added containing the single table.
         """
-        context = super().get_context_data(request, **kwargs)
+        context = {}
         # If the data hasn't been manually loaded before now,
         # make certain it's loaded before setting the context.
         self.load_table_data()
@@ -540,13 +453,3 @@ class TableTab(Tab):
                 context["table"] = table
             context["%s_table" % table_name] = table
         return context
-
-    def has_prev_data(self, table):
-        return False
-
-    def has_more_data(self, table):
-        return False
-
-
-class DetailTabsGroup(TabGroup):
-    template_name = "horizon/common/_detail_tab_group.html"
